package com.iformall.mapper;

import java.util.*;
import com.iformall.common.CommonMapper;
import com.iformall.domain.po.WxScoreHistory;
import com.iformall.domain.vo.WxCreditHistoryVo;
import com.iformall.domain.po.WxCreditHistory;

public interface WxCreditHistoryMapper extends CommonMapper<WxCreditHistory, Long> {

	List<WxCreditHistory> findList(WxCreditHistory wxCreditHistory);

	List<WxCreditHistoryVo> findListMore(WxCreditHistory wxCreditHistory);

	int loginCount(WxCreditHistory wxCreditHistory);

	int creditAmount(Long cUserId);

	int countList(WxCreditHistory wxCreditHistory);

}
